# Node.js Example: Initiating a Withdrawal

This example shows how to initiate a cryptocurrency withdrawal using Node.js.

```javascript
/**
 * Sphere API - Initiate Withdrawal Example
 */

const axios = require('axios');
const crypto = require('crypto');

// API credentials
const merchantId = 'your_merchant_id';
const apiKey = 'your_api_key';

// API endpoint
const apiUrl = 'https://api.coinssend.com/v1/withdrawals';

// Withdrawal data
const withdrawalData = {
  network: 'tron',
  coin: 'usdt',
  amount: '50.75', // Amount in actual value (e.g., 50.75 USDT)
  to_address: 'TXHXiQ2aXYqtz3y9gEGiVjzxXxXechdLwE'
};

// Generate signature
const jsonData = JSON.stringify(withdrawalData);
const base64Data = Buffer.from(jsonData).toString('base64');
const signature = crypto.createHash('md5').update(base64Data + apiKey).digest('hex');

// Make API request
axios({
  method: 'post',
  url: apiUrl,
  headers: {
    'Content-Type': 'application/json',
    'Merchant': merchantId,
    'Sign': signature
  },
  data: withdrawalData
})
.then(response => {
  console.log('Withdrawal initiated successfully!');
  console.log('Withdrawal ID:', response.data.withdrawal_id);
  console.log('Status:', response.data.status);
  console.log('Message:', response.data.message);
})
.catch(error => {
  console.error('Error initiating withdrawal:');
  if (error.response) {
    // The request was made and the server responded with an error status
    console.error('Status:', error.response.status);
    console.error('Data:', error.response.data);
  } else if (error.request) {
    // The request was made but no response was received
    console.error('No response received:', error.request);
  } else {
    // Error setting up the request
    console.error('Request error:', error.message);
  }
});
```

## TypeScript Version

For a TypeScript implementation:

```typescript
import axios, { AxiosResponse, AxiosError } from 'axios';
import * as crypto from 'crypto';

// Define types
interface WithdrawalRequest {
  network: string;
  coin: string;
  amount: string;
  to_address: string;
}

interface WithdrawalResponse {
  success: boolean;
  withdrawal_id: string;
  status: string;
  message: string;
}

interface ErrorResponse {
  status: string;
  error: {
    code: number;
    message: string;
  };
}

// API credentials
const merchantId: string = 'your_merchant_id';
const apiKey: string = 'your_api_key';

// API endpoint
const apiUrl: string = 'https://api.coinssend.com/v1/withdrawals';

// Withdrawal data
const withdrawalData: WithdrawalRequest = {
  network: 'tron',
  coin: 'usdt',
  amount: '50.75',
  to_address: 'TXHXiQ2aXYqtz3y9gEGiVjzxXxXechdLwE'
};

// Generate signature
const jsonData: string = JSON.stringify(withdrawalData);
const base64Data: string = Buffer.from(jsonData).toString('base64');
const signature: string = crypto.createHash('md5').update(base64Data + apiKey).digest('hex');

// Make API request
axios({
  method: 'post',
  url: apiUrl,
  headers: {
    'Content-Type': 'application/json',
    'Merchant': merchantId,
    'Sign': signature
  },
  data: withdrawalData
})
.then((response: AxiosResponse<WithdrawalResponse>) => {
  console.log('Withdrawal initiated successfully!');
  console.log('Withdrawal ID:', response.data.withdrawal_id);
  console.log('Status:', response.data.status);
  console.log('Message:', response.data.message);
})
.catch((error: AxiosError<ErrorResponse>) => {
  console.error('Error initiating withdrawal:');
  if (error.response) {
    console.error('Status:', error.response.status);
    console.error('Error message:', error.response.data?.error?.message || 'Unknown error');
  } else if (error.request) {
    console.error('No response received:', error.request);
  } else {
    console.error('Request error:', error.message);
  }
});
```

## Reusable API Client Class

For multiple API calls, create a reusable client class:

```javascript
const axios = require('axios');
const crypto = require('crypto');

class SphereApiClient {
  constructor(merchantId, apiKey, baseUrl = 'https://api.coinssend.com/v1/') {
    this.merchantId = merchantId;
    this.apiKey = apiKey;
    this.baseUrl = baseUrl;
    this.axiosInstance = axios.create({
      baseURL: this.baseUrl,
      headers: {
        'Content-Type': 'application/json',
        'Merchant': this.merchantId
      }
    });
  }

  async createInvoice(orderId, amount, isCustomerFee = false) {
    const data = {
      order_id: orderId,
      amount: amount,
      is_customer_fee: isCustomerFee
    };
    
    return this.sendRequest('post', 'invoices', data);
  }
  
  async getInvoice(invoiceCode) {
    return this.sendRequest('get', `invoices/${invoiceCode}`);
  }
  
  async initiateWithdrawal(network, coin, amount, toAddress) {
    const data = {
      network: network,
      coin: coin,
      amount: amount,
      to_address: toAddress
    };
    
    return this.sendRequest('post', 'withdrawals', data);
  }
  
  async sendRequest(method, endpoint, data = null) {
    try {
      let headers = {};
      
      if (data) {
        const jsonData = JSON.stringify(data);
        const base64Data = Buffer.from(jsonData).toString('base64');
        const signature = crypto.createHash('md5').update(base64Data + this.apiKey).digest('hex');
        headers.Sign = signature;
      }
      
      const response = await this.axiosInstance({
        method: method,
        url: endpoint,
        headers: headers,
        data: data
      });
      
      return response.data;
    } catch (error) {
      if (error.response) {
        throw new Error(error.response.data?.error?.message || 'API Error');
      } else if (error.request) {
        throw new Error('No response received from API');
      } else {
        throw new Error(`Request failed: ${error.message}`);
      }
    }
  }
}

// Usage
async function run() {
  try {
    const client = new SphereApiClient('your_merchant_id', 'your_api_key');
    
    // Create an invoice
    const invoice = await client.createInvoice(`order_${Date.now()}`, '100.50', true);
    console.log('Invoice created:', invoice.data);
    
    // Initiate a withdrawal
    const withdrawal = await client.initiateWithdrawal('tron', 'usdt', '50.75', 'TXHXiQ2aXYqtz3y9gEGiVjzxXxXechdLwE');
    console.log('Withdrawal initiated:', withdrawal);
  } catch (error) {
    console.error('Error:', error.message);
  }
}

run();
```

## Using with Express.js for Webhook Handling

This example shows how to create a withdrawal client and handle webhooks:

```javascript
const express = require('express');
const bodyParser = require('body-parser');
const crypto = require('crypto');
const axios = require('axios');

// Initialize Express app
const app = express();
app.use(bodyParser.json());

// API credentials
const merchantId = 'your_merchant_id';
const apiKey = 'your_api_key';

// Webhook verification function
function verifyWebhookSignature(payload, signature) {
  const computedSignature = crypto.createHmac('sha256', apiKey)
    .update(JSON.stringify(payload))
    .digest('hex');
  
  return crypto.timingSafeEqual(
    Buffer.from(computedSignature),
    Buffer.from(signature)
  );
}

// Function to initiate a withdrawal
async function initiateWithdrawal(network, coin, amount, toAddress) {
  const withdrawalData = { network, coin, amount, to_address: toAddress };
  const jsonData = JSON.stringify(withdrawalData);
  const base64Data = Buffer.from(jsonData).toString('base64');
  const signature = crypto.createHash('md5').update(base64Data + apiKey).digest('hex');
  
  try {
    const response = await axios({
      method: 'post',
      url: 'https://api.coinssend.com/v1/withdrawals',
      headers: {
        'Content-Type': 'application/json',
        'Merchant': merchantId,
        'Sign': signature
      },
      data: withdrawalData
    });
    
    return response.data;
  } catch (error) {
    console.error('Withdrawal error:', error.response?.data || error.message);
    throw new Error(error.response?.data?.error?.message || 'Withdrawal failed');
  }
}

// Webhook endpoint
app.post('/webhook', (req, res) => {
  const signature = req.headers['x-signature'];
  
  // Verify signature
  if (!signature || !verifyWebhookSignature(req.body, signature)) {
    console.error('Invalid webhook signature');
    return res.status(401).json({ error: 'Invalid signature' });
  }
  
  // Process the webhook event
  const event = req.body;
  console.log('Received webhook:', event.type);
  
  // Handle different event types
  switch (event.type) {
    case 'withdrawal.success':
      console.log('Withdrawal completed:', event.data.withdrawal_id);
      // Update your database or notify your systems
      break;
    case 'withdrawal.failed':
      console.log('Withdrawal failed:', event.data.withdrawal_id, event.data.error);
      // Handle the failure - maybe retry or notify support
      break;
    // Handle other event types...
  }
  
  // Always respond with 200 to acknowledge receipt
  res.status(200).json({ received: true });
});

// Example API endpoint to create a withdrawal request
app.post('/api/withdraw', async (req, res) => {
  try {
    const { network, coin, amount, toAddress } = req.body;
    
    // Validate request parameters
    if (!network || !coin || !amount || !toAddress) {
      return res.status(400).json({ error: 'Missing required parameters' });
    }
    
    // Initiate the withdrawal
    const result = await initiateWithdrawal(network, coin, amount, toAddress);
    
    res.json({
      success: true,
      withdrawal_id: result.withdrawal_id,
      status: result.status
    });
  } catch (error) {
    res.status(400).json({
      success: false,
      error: error.message
    });
  }
});

// Start the server
const PORT = process.env.PORT || 3000;
app.listen(PORT, () => {
  console.log(`Server running on port ${PORT}`);
});
```
